package ch.std.genericdto.rest;

import static org.junit.jupiter.api.Assertions.assertNotNull;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;

import ch.std.genericdto.dto.RequestDTO;
import ch.std.genericdto.dto.ResponseDTO;
import ch.std.genericdto.rest.EchoController.Echo;

@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
public class EchoControllerTests {

	Logger logger = LoggerFactory.getLogger(EchoControllerTests.class);

	@Autowired
	private TestRestTemplate restTemplate;

	@BeforeEach
	public void setup() {
		logger.info("EchoControllerTests.setup");
	}

	@Test
	public void testGetEcho() throws Exception {
		String url = "/rest/echo?value=Bern";
		ResponseEntity<ResponseDTO<Echo>> responseDTO = this.restTemplate.exchange(url, HttpMethod.GET, null, new ParameterizedTypeReference<ResponseDTO<Echo>>() {});
		assertNotNull(responseDTO);
		assertNotNull(responseDTO.getBody());
	}

	@Test
	public void testPostEcho() throws Exception {
		String url = "/rest/echo";
		RequestDTO<Echo> requestDTO = new RequestDTO<Echo>();
		requestDTO.setHeadValue("name", "James");
		requestDTO.setBody(new Echo("James"));
		HttpEntity<RequestDTO<Echo>> httpEntityRequest = new HttpEntity<>(requestDTO);
		ResponseEntity<ResponseDTO<Echo>> responseDTO = this.restTemplate.exchange(url, HttpMethod.POST, httpEntityRequest, new ParameterizedTypeReference<ResponseDTO<Echo>>() {});
		assertNotNull(responseDTO);
		assertNotNull(responseDTO.getBody());
	}

	@Test
	public void testPostEcho2() throws Exception {
		String url = "/rest/echo/real";
		RequestDTO<Echo> requestDTO = new RequestDTO<Echo>();
		requestDTO.setHeadValue("name", "James");
		requestDTO.setBody(new Echo("James"));
		HttpEntity<RequestDTO<Echo>> httpEntityRequest = new HttpEntity<>(requestDTO);		
		ResponseEntity<ResponseDTO<Echo>> responseDTO = this.restTemplate.exchange(url, HttpMethod.POST, httpEntityRequest, new ParameterizedTypeReference<ResponseDTO<Echo>>() {});
		assertNotNull(responseDTO);
		assertNotNull(responseDTO.getBody());
	}

}
